<%@page import="java.io.*,
                java.util.*,
                blackboard.base.*,
                blackboard.data.*,
                blackboard.data.course.*,
                blackboard.data.user.*,
                blackboard.persist.*,
                blackboard.persist.course.*,
                blackboard.persist.user.*,
                blackboard.platform.*,
                blackboard.platform.filesystem.*,
                blackboard.platform.plugin.*,
                blackboard.platform.session.*,
                com.webassign.*"
        errorPage="/error.jsp"
%>

<%@ taglib uri="/bbData" prefix="bbData"%>
<%@ taglib uri="/bbUI" prefix="bbUI"%>

<bbData:context id="ctx">
<%
	BbSession bbSession = BbServiceManager.getSessionManagerService().getSession( request );
	Course bbcourse = ctx.getCourse();
	BbPersistenceManager bbPm = BbServiceManager.getPersistenceService().getDbPersistenceManager();

	CourseMembershipDbLoader cmLoader = (CourseMembershipDbLoader) bbPm.getLoader( CourseMembershipDbLoader.TYPE );
	UserDbLoader uLoader = (UserDbLoader) bbPm.getLoader( UserDbLoader.TYPE );
	BbList userList = uLoader.loadByCourseId(bbcourse.getId());
	ListIterator userIterator = userList.listIterator();

	Hashtable roles = new Hashtable();
	CourseMembership.Role[] allowedRoles = {
		CourseMembership.Role.INSTRUCTOR,
		CourseMembership.Role.TEACHING_ASSISTANT,
		CourseMembership.Role.GRADER,
		CourseMembership.Role.COURSE_BUILDER
	};
	String[] roleCheckboxes = {
		"instructor",
		"teaching_assistant",
		"grader",
		"course_builder"
	};
	for (int i = 0; i < allowedRoles.length; i++) {
		roles.put(allowedRoles[i],roleCheckboxes[i]);
	}

	int bbstudentCount = 0;
	List bbstudents = new ArrayList();
	while (userIterator.hasNext()) {
		User u = (User) userIterator.next();
		try {
			CourseMembership cm = cmLoader.loadByCourseAndUserId(bbcourse.getId(),u.getId());
			if (cm.getRole() == CourseMembership.Role.STUDENT) {
				bbstudents.add(bbstudentCount,u);
				bbstudentCount++;
			}
			else if (roles.containsKey(cm.getRole())) {
				String roleParam = "include_" + roles.get(cm.getRole()).toString();
				if ((request.getParameter(roleParam) != null) && (request.getParameter(roleParam).equals("true"))) {
					bbstudents.add(bbstudentCount,u);
					bbstudentCount++;
				}
			}
		} catch (Exception e) {
		}
	}

	FileSystemService fileSysService = (FileSystemService)
		BbServiceManager.lookupService(blackboard.platform.filesystem.FileSystemService.class);
	File courseContentDir = fileSysService.getCourseDirectory(bbcourse);
	File courseConfigFile = new File(courseContentDir,com.webassign.Props.propFileName());

	boolean linkedCourse = false;
	com.webassign.Props wacourseprops = new com.webassign.Props(courseConfigFile);
	String wacourseID = wacourseprops.getProp("course.id");
	if (wacourseID.length() > 0) linkedCourse = true;

	int added = 0;
	int updated = 0;
	int dropped = 0;

	boolean fault = false;
	boolean cerror = false;

	if (linkedCourse) {
		File configDir = PlugInUtil.getConfigDirectory(com.webassign.Props.vid(),com.webassign.Props.handle());
		File configFile = new File(configDir,com.webassign.Props.propFileName());
		com.webassign.Props waprops = new com.webassign.Props(configFile);
		String institutionCode = waprops.getProp("institution.code");
		String institutionKey = waprops.getProp("institution.key");

		com.webassign.SOAP warosterSOAP = new com.webassign.SOAP("syncRoster");
		warosterSOAP.addElem("courseID",wacourseprops.getProp("course.id"));
		warosterSOAP.addElem("institution",institutionCode);

		for (int i = 0; i < bbstudents.size(); i++) {
			User u = (User) bbstudents.get(i);
			warosterSOAP.addElem("username",u.getUserName());
			warosterSOAP.addElem("fullname",u.getFamilyName() + ", " + u.getGivenName() + " " + u.getMiddleName());
			warosterSOAP.addElem("email",u.getEmailAddress());
			warosterSOAP.addElem("index",Integer.toString(i));
		}
		warosterSOAP.addElem("drop",request.getParameter("wadrop"));

		warosterSOAP.sendCall(institutionKey);

		if (warosterSOAP.droppedCall()) {
			fault = true;
		}
		else if (warosterSOAP.getResultByKey("result").equals("success")) {
			added = Integer.parseInt(warosterSOAP.getResultByKey("added"));
			updated = Integer.parseInt(warosterSOAP.getResultByKey("updated"));
			dropped = Integer.parseInt(warosterSOAP.getResultByKey("dropped"));
		}
		else {
			cerror = true;
		}
	}

	String mainHref = "main.jsp?course_id=" + ctx.getCourseId().toExternalString();
%>

<bbUI:docTemplate title="Sync Roster">
<bbUI:coursePage courseId="<%=bbSession.lookupCourseContext(request).getCourseId()%>">

<bbUI:breadcrumbBar handle="control_panel">
	<bbUI:breadcrumb href="<%=mainHref%>"> WebAssign</bbUI:breadcrumb>
	<bbUI:breadcrumb> Sync Roster</bbUI:breadcrumb>
</bbUI:breadcrumbBar>

<% if (linkedCourse) { %>
<%   if (fault || cerror) { %>
<bbUI:receipt type="FAIL">
There was an error processing this request. Please try again later.
</bbUI:receipt>
<%   } else { %>
<bbUI:receipt type="SUCCESS" recallUrl="<%=mainHref%>">
<p>
<%     if ((added > 0) || (updated > 0) || (dropped > 0)) { %>
<%       if (added > 0) { %>
Added <%=added%> user(s)<br>
<%       } %>
<%       if (updated > 0) { %>
Changed information about <%=updated%> user(s)<br>
<%       } %>
<%       if (dropped > 0) { %>
Dropped <%=dropped%> user(s)
<%       } %>
<%     } else { %>
No changes were made to your WebAssign roster.
<%     } %>
</p>
</bbUI:receipt>
<%   } %>
<% } else { %>
<p>Your course is not linked to an existing WebAssign course.</p>
<div align="right">
<bbUI:button type="FORM_ACTION" name="ok" alt="OK" action="LINK" targetUrl="<%=mainHref%>" />
</div>
<% } %>

</bbUI:coursePage>
</bbUI:docTemplate>

</bbData:context>
